
//Incluir la librería para controlar los servos
#include "ServoEasing.h"
ServoEasing servoTop;
ServoEasing servoBottom;
int location = 31;

//Definir los ángulos de ambos servos abiertos y cerrados
int bottom_closed = 140;
int top_closed = 180;
int bottom_open = 35;
int top_open = 35;

//Incluir librería y definir pin para controlar el sensor DHT11
#include <DHT.h>
#define DHTPIN 5
#define DHTTYPE DHT11
DHT dht(DHTPIN, DHTTYPE);

//Incluir librería y definir tamaño para controlar pantalla oled
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#define ancho 128
#define alto 64
#define OLED_RESET 4
Adafruit_SSD1306 oled(ancho, alto, &Wire, OLED_RESET);

//Definir variables y pin para medir la carga de las baterías
#define lectura A0
float vo = 0.0;
float vi = 0.0;
float r1 = 100000.0;
float r2 = 10000.0;
int carga;

//Definir los pines que se utilizaran para los leds 
#define led1 2
#define led2 3

//Definir pines y variables para controlar el sensor ultrasónico
#define Pecho 6
#define Ptrig 7
int duracion;
int distancia;


void setup() {
   dht.begin(); //Inicializar el sensor DHT11
  
   pinMode(lectura, INPUT); 
   
   Wire.begin();
   oled.begin(SSD1306_SWITCHCAPVCC, 0x3c); //Inicializar la pantalla oled

   //Inicializar los pines de ambos leds y del sensor ultrasónico en el modo necesario
   pinMode(Pecho, INPUT);     
   pinMode(Ptrig, OUTPUT);   
   pinMode(led1,OUTPUT);
   pinMode(led2,OUTPUT);

   //Definir servos y especificaciones de funcionamiento
   servoTop.attach(9);
   servoBottom.attach(10);
   setSpeedForAllServos(190);
   servoTop.setEasingType(EASE_CUBIC_IN_OUT);
   servoBottom.setEasingType(EASE_CUBIC_IN_OUT);
   synchronizeAllServosStartAndWaitForAllServosToStop();

   //Abrir el casco
   servoTop.setEaseTo(top_open);
   servoBottom.setEaseTo(bottom_open);
   synchronizeAllServosStartAndWaitForAllServosToStop();
   delay(600);
   
}

void loop() {
  dist(); //Utilizar la función dist
  pantalla(); //Utilizar la función pantalla
  cargabat(); //Utilizar la función cargabat
  mascara(); //Utilizar la función mascara
}


//Función temperatura
void temperatura(){
  float t = dht.readTemperature(); //Leer la temperatura en grados centígrados
  delay(500); //Esperar medio segundo
  }

//Función carga bat
void cargabat(){
  int value = analogRead(lectura); //Leer el valor analógico del pin
  vo = (value * 5.0)/1024;  //Calcular el voltaje de salido del divisor de tensión
  vi = vo / (r2/(r1+r2));   //Calcular el voltaje de entrada del divisor de tensión
  float d = vi - 4;        
  int c = d * 100;           //Calcular un porcentaje de carga de la batería basandose en la disminución del voltaje
  
  }


//Función pantalla
void pantalla(){
   float t = dht.readTemperature();
   delay(500);
   oled.clearDisplay();
   oled.setTextColor(WHITE);
   oled.setCursor(20,4);
   oled.setTextSize(2);
   oled.print("Mark II");     //Mostrar en pantalla el nombre del proyecto
   oled.drawRect(16,0, 88,22, WHITE);
   oled.setCursor(5,35);
   oled.setTextSize(1);
   oled.print("Reactor");
   oled.setCursor(13,50);
   oled.setTextSize(1);
   oled.print(c);           //Mostrar en pantalla la carga de las baterías
   oled.setCursor(48,50);
   oled.setTextSize(1);
   oled.print("|");
   oled.setCursor(60,35);
   oled.setTextSize(1);
   oled.print("Temperature");
   oled.setCursor(70,50);
   oled.setTextSize(1);
   oled.print(t);            //Mostrar en pantalla la temperatura del casco
   oled.setCursor(30,50);
   oled.setTextSize(1);
   oled.print("%");
   oled.setCursor(105,50);
   oled.setTextSize(1);
   oled.print("C");
   oled.display();
    
   
  }


//Función distancia
void dist(){
    //Medir la distancia hasta el objeto más cercano
    delay(250);
    digitalWrite(Ptrig, HIGH);
    delay(1);
    digitalWrite(Ptrig, LOW);
    duracion = pulseIn(Pecho, HIGH);
    distancia = duracion / 58.2;
    Serial.println(distancia);
    delay(250);
  
  }


//Función máscara
void mascara(){
  //Si la distancia es entre 0 y 5 cm 
  if (distancia <=5 && distancia >= 0){
    //Si la más cara está cerrada
    if (location > bottom_open) {
     //Abrir la mascara y apagar los leds
     servoTop.setEaseTo(top_open);
     servoBottom.setEaseTo(bottom_open);
     synchronizeAllServosStartAndWaitForAllServosToStop();
     location = bottom_open;
     digitalWrite(led1, LOW);
     digitalWrite(led2, LOW);
     delay(600);
     } 
     //Si la más cara está abierta
     else {
      //Cerrar la mascara y encender los leds
     servoTop.setEaseTo(top_closed);
     servoBottom.setEaseTo(bottom_closed);
     synchronizeAllServosStartAndWaitForAllServosToStop();
     location = bottom_closed;
     digitalWrite(led1, HIGH);
     digitalWrite(led2, HIGH);
     delay(600);
     }
    }
  }
